@@ -17,7 +17,16 @@ import numpy as np
 import unicodedata
 from unidecode import unidecode
 
-from ..data.categories import title_map, root_map, ing_keys, ing_map, exceptions
+from ..data.categories import (
+    title_map,
+    root_map,
+    ing_keys,
+    ing_map,
+    exceptions,
+    function_map,
+)
+
+from ..data.categories import proteins
 
 # from data.categories import title_map, root_map, ing_keys, ing_map, exceptions
 import spacy
@@ -50,6 +59,7 @@ class RecipeReader:
         phrase = re.sub(r"(?<=[0-9]) *\. *(?=[0-9])", ".", phrase)
         phrase = re.sub(r" '", "'", phrase)
         phrase = re.sub(r"(,[^,]+)?< ?a href.*", "", phrase)
+        phrase = re.sub(r""" *<(?:"[^"]*"['"]*|'[^']*'['"]*|[^'">])+> *""", "", phrase)
         phrase = re.sub(r"(?<=[a-z])/[a-z]+", "", phrase)
         phrase = re.sub(r"\b(?:5|five)[- ]?spice", "fivespice", phrase)
         phrase = re.sub(r".*: ?", "", phrase)
@@ -57,6 +67,23 @@ class RecipeReader:
         phrase = phrase.strip()
         return phrase
 
+    def merge_ingredients(self, ingredients):
+        out = []
+        outIngs = []
+
+        for ing in ingredients:
+            if ing["simple"] not in outIngs:
+                out += [ing.copy()]
+                outIngs += [ing["simple"]]
+            else:
+                for i, o in enumerate(out):
+                    if o["simple"] == ing["simple"] and o["unit"] == ing["unit"]:
+                        if not ing["quantity"] or not o["quantity"]:
+                            continue
+                        out[i]["quantity"] += ing["quantity"]
+
+        return out
+
     def read_phrase(self, phrase):
         if not P_filter(phrase):
             return None
@@ -73,12 +100,13 @@ class RecipeReader:
 
         values = re.search(re_.INGREDIENT, phrase).groupdict()
 
+        values["unit"] = None
         if values["quantity"]:
             values["quantity"], values["unit"] = re.search(
                 rf"(?P<quantity>{re_.Q})? ?(?P<unit>.*)?", values["quantity"]
             ).groups()
             values["quantity"] = Q_to_number(values["quantity"])
-        values["unit"] = None
+
         values["unit"] = U_unify(values["unit"])
         values["quantity"], values["unit"] = Q_U_unify(
             values["quantity"], values["unit"]
@@ -86,13 +114,18 @@ class RecipeReader:
 
         values["size"] = S_unify(values["size"])
 
-        if values["ingredient"] != values["ingredient"]:
+        if values["ingredient"] != values["ingredient"] or not values["ingredient"]:
             return None
 
         values["ingredient"] = I_to_singular(values["ingredient"])
         values["simple"] = I_label_protein(values["ingredient"])
         values["simple"] = I_simplify(values["simple"])
 
+        if values["simple"] == "sugar":
+            values["quantity"], values["unit"] = Q_U_sugar(
+                values["quantity"], values["unit"]
+            )
+
         filtered = {
             c: values[c]
             for c in ["quantity", "unit", "size", "color", "ingredient", "simple"]
@@ -119,7 +152,7 @@ class RecipeReader:
         title = re.sub(r"\bnan\b", r"\bnaan\b", title)
         return title
 
-    def categorize(self, title, ingredients, steps):
+    def categorize(self, title, ingredients, steps, parsedPhrases):
         categories = []
         roots = [
             token.text.lower()
@@ -133,6 +166,7 @@ class RecipeReader:
             for ing in ingredients:
                 if key == ing:
                     categories.append(key)
+                    break
 
         for key, regex in ing_map.items():
             for ing in ingredients:
@@ -156,9 +190,15 @@ class RecipeReader:
         for key, regex in title_map.items():
             if re.search(regex, title):
                 categories.append(key)
-                break
 
-        return categories
+        for key, labelF in function_map.items():
+            if labelF(parsedPhrases):
+                categories.append(key)
+
+        if any(p in categories for p in proteins):
+            categories = [c for c in categories if c != "vegetarian"]
+
+        return list(set(categories))
 
     def categories_to_models(self, categories_a):
         out = []
@@ -169,14 +209,21 @@ class RecipeReader:
 
         return out if len(out) > 0 else ["other"]
 
-    def read(self, title, phrases, steps):
-        ingredients = [self.read_phrase(p)["simple"] for p in phrases]
+    def read(self, title, phrases, steps=[]):
+        parsedPhrases = [self.read_phrase(p) for p in phrases]
+        parsedPhrases = self.merge_ingredients([p for p in parsedPhrases if p])
         columns = sorted(list(set(list(ing_funnel.values()))))
-        ingredients = [self.funnel(p) for p in ingredients if self.funnel(p)]
-        categories = self.categorize(title, ingredients, steps)
+
+        ingredients = [
+            self.funnel(p["simple"]) for p in parsedPhrases if self.funnel(p["simple"])
+        ]
+        categories = self.categorize(title, ingredients, steps, parsedPhrases)
+        if "niche" in categories:
+            categories = ["niche"]
 
         return {
             "ingredients": ingredients,
+            "ingredients_": parsedPhrases,
             "title": title,
             "categories": categories,
             "models": self.categories_to_models(categories),
